@@ -49,19 +49,31 @@ struct EmulateFp8ExtTruncPass final
 };
 
 struct Fp8ExtToTableLookupPattern final : public OpConversionPattern<ExtFOp> {
+  using OpConversionPattern<ExtFOp>::OpConversionPattern;
+
   bool hasF8ConversionInstrs = false;
   bool hasOcpF8ConversionInstrs = false;
   Fp8ExtToTableLookupPattern(MLIRContext *ctx, bool hasF8ConversionInstrs,
                              bool hasOcpF8ConversionInstrs)
-      : OpConversionPattern<ExtFOp>::OpConversionPattern(ctx),
-        hasF8ConversionInstrs(hasF8ConversionInstrs),
+      : OpConversionPattern(ctx), hasF8ConversionInstrs(hasF8ConversionInstrs),
         hasOcpF8ConversionInstrs(hasOcpF8ConversionInstrs) {}
-  LogicalResult match(ExtFOp op) const override;
+  LogicalResult match(ExtFOp op) const;
   void rewrite(ExtFOp op, OpAdaptor adaptor,
-               ConversionPatternRewriter &rewriter) const override;
+               ConversionPatternRewriter &rewriter) const;
+  LogicalResult
+  matchAndRewrite(ExtFOp op, OpAdaptor adaptor,
+                  ConversionPatternRewriter &rewriter) const override {
+    auto result = match(op);
+    if (result.succeeded()) {
+      rewrite(op, adaptor, rewriter);
+    }
+    return result;
+  }
 };
 
 struct Fp8TruncToCallPattern final : public OpConversionPattern<TruncFOp> {
+  using OpConversionPattern::OpConversionPattern;
+
   FlatSymbolRefAttr f8E4M3FNUZFunc;
   FlatSymbolRefAttr f8E5M2FNUZFunc;
   FlatSymbolRefAttr f8E4M3FNFunc; // OCP
@@ -77,16 +89,25 @@ struct Fp8TruncToCallPattern final : public OpConversionPattern<TruncFOp> {
                         FlatSymbolRefAttr f8E5M2Func,
                         bool hasF8ConversionInstrs,
                         bool hasOcpF8ConversionInstrs)
-      : OpConversionPattern<TruncFOp>::OpConversionPattern(ctx),
-        f8E4M3FNUZFunc(f8E4M3FNUZFunc), f8E5M2FNUZFunc(f8E5M2FNUZFunc),
-        f8E4M3FNFunc(f8E4M3FNFunc), f8E5M2Func(f8E5M2Func),
-        hasF8ConversionInstrs(hasF8ConversionInstrs),
+      : OpConversionPattern(ctx), f8E4M3FNUZFunc(f8E4M3FNUZFunc),
+        f8E5M2FNUZFunc(f8E5M2FNUZFunc), f8E4M3FNFunc(f8E4M3FNFunc),
+        f8E5M2Func(f8E5M2Func), hasF8ConversionInstrs(hasF8ConversionInstrs),
         hasOcpF8ConversionInstrs(hasOcpF8ConversionInstrs) {}
 
-  LogicalResult match(TruncFOp op) const override;
+  LogicalResult match(TruncFOp op) const;
   void rewrite(TruncFOp op, OpAdaptor adaptor,
-               ConversionPatternRewriter &rewriter) const override;
+               ConversionPatternRewriter &rewriter) const;
+  LogicalResult
+  matchAndRewrite(TruncFOp op, OpAdaptor adaptor,
+                  ConversionPatternRewriter &rewriter) const override {
+    auto result = match(op);
+    if (result.succeeded()) {
+      rewrite(op, adaptor, rewriter);
+    }
+    return result;
+  }
 };
+
 } // namespace
 
 static bool isFp8(Type t) {
@@ -165,8 +186,8 @@ static Value getFloatValueTableFor(Type elementType, Operation *op,
   ElementsAttr tableElemsAttr = DenseElementsAttr::get<float>(
       RankedTensorType::get(256, rewriter.getF32Type()), tableElems);
   OpBuilder nowhereBuilder(module->getContext(), rewriter.getListener());
-  table = nowhereBuilder.create<memref::GlobalOp>(
-      op->getLoc(), extTableName,
+  table = memref::GlobalOp::create(
+      nowhereBuilder, op->getLoc(), extTableName,
       /*sym_visibility=*/rewriter.getStringAttr("private"),
       /*type=*/globalType,
       /*initial_value=*/tableElemsAttr,
@@ -188,12 +209,12 @@ void Fp8ExtToTableLookupPattern::rewrite(
 
   Value table = getFloatValueTableFor(elemType, op, rewriter);
   auto oneToFloat = [&](Value fp8) -> Value {
-    Value bitcast = rewriter.create<BitcastOp>(loc, rewriter.getI8Type(), fp8);
+    Value bitcast = BitcastOp::create(rewriter, loc, rewriter.getI8Type(), fp8);
     // Don't sign-extend the byte when index casting.
-    Value i32 = rewriter.create<ExtUIOp>(loc, rewriter.getI32Type(), bitcast);
+    Value i32 = ExtUIOp::create(rewriter, loc, rewriter.getI32Type(), bitcast);
     Value index =
-        rewriter.create<IndexCastOp>(loc, rewriter.getIndexType(), i32);
-    Value extended = rewriter.create<memref::LoadOp>(loc, table, index);
+        IndexCastOp::create(rewriter, loc, rewriter.getIndexType(), i32);
+    Value extended = memref::LoadOp::create(rewriter, loc, table, index);
     return extended;
   };
 
@@ -201,9 +222,9 @@ void Fp8ExtToTableLookupPattern::rewrite(
     if (outElemType.isF32())
       return floats;
     if (outElemType.getIntOrFloatBitWidth() < 32)
-      return rewriter.create<TruncFOp>(loc, outType, floats);
+      return TruncFOp::create(rewriter, loc, outType, floats);
     if (outElemType.getIntOrFloatBitWidth() > 32)
-      return rewriter.create<ExtFOp>(loc, outType, floats);
+      return ExtFOp::create(rewriter, loc, outType, floats);
     llvm_unreachable("f32 is the only 32-bit float type");
   };
   auto inVecType = dyn_cast<VectorType>(inType);
@@ -221,9 +242,9 @@ void Fp8ExtToTableLookupPattern::rewrite(
   for (int64_t i = 0, e = inVecType.getNumElements(); i < e; ++i) {
     SmallVector<int64_t> idx = delinearize(i, strides);
     Value scalar =
-        rewriter.create<vector::ExtractOp>(loc, adaptor.getIn(), idx);
+        vector::ExtractOp::create(rewriter, loc, adaptor.getIn(), idx);
     Value extended = oneToFloat(scalar);
-    floats = rewriter.create<vector::InsertOp>(loc, extended, floats, idx);
+    floats = vector::InsertOp::create(rewriter, loc, extended, floats, idx);
   }
   Value ret = floatsToResult(floats);
   return rewriter.replaceOp(op, ret);
@@ -277,76 +298,76 @@ static FlatSymbolRefAttr makeFp8TruncFunction(Location loc, FloatType outType,
   Block *ret = new Block();
   ret->addArgument(outType, loc);
 
-  Value bits = b.create<BitcastOp>(i32, in);
+  Value bits = BitcastOp::create(b, i32, in);
   const llvm::fltSemantics &outSem = outType.getFloatSemantics();
 
-  Value and1 = b.create<AndIOp>(bits, i32Const((1u << 23u) - 1));
-  Value shr = b.create<ShRUIOp>(bits, i32Const(23));
-  Value and2 = b.create<AndIOp>(shr, i32Const(0xff));
-  Value ir1 = b.create<ShRUIOp>(bits, i32Const(24));
-  Value shl = b.create<AndIOp>(ir1, i32Const(128));
+  Value and1 = AndIOp::create(b, bits, i32Const((1u << 23u) - 1));
+  Value shr = ShRUIOp::create(b, bits, i32Const(23));
+  Value and2 = AndIOp::create(b, shr, i32Const(0xff));
+  Value ir1 = ShRUIOp::create(b, bits, i32Const(24));
+  Value shl = AndIOp::create(b, ir1, i32Const(128));
   Value infNanConst = i32Const(0x7f800000);
-  Value and4 = b.create<AndIOp>(bits, infNanConst);
-  Value cmp = b.create<CmpIOp>(CmpIPredicate::eq, and4, infNanConst);
+  Value and4 = AndIOp::create(b, bits, infNanConst);
+  Value cmp = CmpIOp::create(b, CmpIPredicate::eq, and4, infNanConst);
 
   Block *notInfNan = func.addBlock();
-  Value outNan = b.create<ConstantFloatOp>(APFloat::getQNaN(outSem), outType);
-  b.create<cf::CondBranchOp>(cmp, ret, ValueRange{outNan}, notInfNan,
-                             ValueRange{});
+  Value outNan = ConstantFloatOp::create(b, outType, APFloat::getQNaN(outSem));
+  cf::CondBranchOp::create(b, cmp, ret, ValueRange{outNan}, notInfNan,
+                           ValueRange{});
   b.setInsertionPointToStart(notInfNan);
 
   // A deviation from the MIGraphX: denormals are zero here
-  Value cmp5 = b.create<CmpIOp>(CmpIPredicate::eq, and2, i32Const(0));
-  Value outZero = b.create<ConstantFloatOp>(APFloat::getZero(outSem), outType);
+  Value cmp5 = CmpIOp::create(b, CmpIPredicate::eq, and2, i32Const(0));
+  Value outZero = ConstantFloatOp::create(b, outType, APFloat::getZero(outSem));
   Block *notZero = func.addBlock();
-  b.create<cf::CondBranchOp>(cmp5, ret, ValueRange{outZero}, notZero,
-                             ValueRange{});
+  cf::CondBranchOp::create(b, cmp5, ret, ValueRange{outZero}, notZero,
+                           ValueRange{});
   b.setInsertionPointToStart(notZero);
 
   // For some reason, this is off by one
   uint32_t mBits = outType.getFPMantissaWidth() - 1;
   uint32_t eBits = 7 - mBits;
-  Value sub = b.create<AddIOp>(and2, i32Const(-127));
+  Value sub = AddIOp::create(b, and2, i32Const(-127));
   Value reducedConst1 = i32Const(127 - ((1 << (eBits - 1)) - 2));
-  Value cmp8 = b.create<CmpIOp>(CmpIPredicate::ult, and2, reducedConst1);
+  Value cmp8 = CmpIOp::create(b, CmpIPredicate::ult, and2, reducedConst1);
   Value reducedConst2 = i32Const(127 - ((1 << (eBits - 1)) - 1));
-  Value sub10 = b.create<SubIOp>(reducedConst2, and2);
-  Value exponentDiff0 = b.create<SelectOp>(cmp8, sub10, i32Const(0));
+  Value sub10 = SubIOp::create(b, reducedConst2, and2);
+  Value exponentDiff0 = SelectOp::create(b, cmp8, sub10, i32Const(0));
 
-  Value add12 = b.create<OrIOp>(and1, i32Const(1 << 23));
-  Value ir2 = b.create<MinUIOp>(exponentDiff0, i32Const(15 - eBits));
+  Value add12 = OrIOp::create(b, and1, i32Const(1 << 23));
+  Value ir2 = MinUIOp::create(b, exponentDiff0, i32Const(15 - eBits));
   Value notmaskConst = i32Const(~((1 << (16 + eBits)) - 1));
-  Value notmask = b.create<ShLIOp>(notmaskConst, ir2);
-  Value sub16 = b.create<XOrIOp>(notmask, i32Const(-1));
-  Value and17 = b.create<AndIOp>(add12, sub16);
-  Value sub21 = b.create<AddIOp>(exponentDiff0, i32Const(15 + eBits));
-  Value sroaSpeculataed140 = b.create<MinUIOp>(sub21, i32Const(31));
-  Value shl23 = b.create<ShLIOp>(i32Const(1), sroaSpeculataed140);
-  Value cmp24 = b.create<CmpIOp>(CmpIPredicate::eq, and17, shl23);
+  Value notmask = ShLIOp::create(b, notmaskConst, ir2);
+  Value sub16 = XOrIOp::create(b, notmask, i32Const(-1));
+  Value and17 = AndIOp::create(b, add12, sub16);
+  Value sub21 = AddIOp::create(b, exponentDiff0, i32Const(15 + eBits));
+  Value sroaSpeculataed140 = MinUIOp::create(b, sub21, i32Const(31));
+  Value shl23 = ShLIOp::create(b, i32Const(1), sroaSpeculataed140);
+  Value cmp24 = CmpIOp::create(b, CmpIPredicate::eq, and17, shl23);
   Value cmp25 =
-      b.create<CmpIOp>(CmpIPredicate::sgt, exponentDiff0, i32Const(0));
-  Value sroaSpeculated = b.create<MinUIOp>(exponentDiff0, i32Const(31));
-  Value shr30 = b.create<SelectOp>(cmp25, sroaSpeculated, i32Const(0));
-  Value mantissa0 = b.create<ShRUIOp>(add12, shr30);
+      CmpIOp::create(b, CmpIPredicate::sgt, exponentDiff0, i32Const(0));
+  Value sroaSpeculated = MinUIOp::create(b, exponentDiff0, i32Const(31));
+  Value shr30 = SelectOp::create(b, cmp25, sroaSpeculated, i32Const(0));
+  Value mantissa0 = ShRUIOp::create(b, add12, shr30);
 
-  Value add40 = b.create<AddIOp>(sub, exponentDiff0);
-  Value and38 = b.create<ShRUIOp>(mantissa0, i32Const(23));
-  Value ir3 = b.create<OrIOp>(and38, i32Const(-2));
-  Value add41 = b.create<AddIOp>(add40, ir3);
-  Value sub43 = b.create<AddIOp>(add41, i32Const((1 << (eBits - 1)) + 1));
-  Value and44 = b.create<ShRUIOp>(mantissa0, i32Const(16 + eBits));
-  Value ir4 = b.create<AndIOp>(and44, i32Const(1));
+  Value add40 = AddIOp::create(b, sub, exponentDiff0);
+  Value and38 = ShRUIOp::create(b, mantissa0, i32Const(23));
+  Value ir3 = OrIOp::create(b, and38, i32Const(-2));
+  Value add41 = AddIOp::create(b, add40, ir3);
+  Value sub43 = AddIOp::create(b, add41, i32Const((1 << (eBits - 1)) + 1));
+  Value and44 = ShRUIOp::create(b, mantissa0, i32Const(16 + eBits));
+  Value ir4 = AndIOp::create(b, and44, i32Const(1));
   Value resolvedConst3 = i32Const((1 << (16 + eBits)) - 1);
-  Value sext = b.create<AddIOp>(ir4, resolvedConst3);
-  Value cond51 = b.create<SelectOp>(cmp24, sext, i32Const(0));
-  Value cond54 = b.create<AddIOp>(cond51, mantissa0);
-  Value and55 = b.create<AndIOp>(cond54, resolvedConst3);
-  Value add56 = b.create<AddIOp>(and55, mantissa0);
-  Value cmp57 = b.create<CmpIOp>(CmpIPredicate::ne, sub43, i32Const(0));
-  Value and58 = b.create<AndIOp>(add56, i32Const(1 << 23));
-  Value tobool59Not = b.create<CmpIOp>(CmpIPredicate::eq, and58, i32Const(0));
-  Value trueConst = b.create<ConstantIntOp>(true, 1);
-  Value brCond133 = b.create<SelectOp>(cmp57, trueConst, tobool59Not);
+  Value sext = AddIOp::create(b, ir4, resolvedConst3);
+  Value cond51 = SelectOp::create(b, cmp24, sext, i32Const(0));
+  Value cond54 = AddIOp::create(b, cond51, mantissa0);
+  Value and55 = AndIOp::create(b, cond54, resolvedConst3);
+  Value add56 = AddIOp::create(b, and55, mantissa0);
+  Value cmp57 = CmpIOp::create(b, CmpIPredicate::ne, sub43, i32Const(0));
+  Value and58 = AndIOp::create(b, add56, i32Const(1 << 23));
+  Value tobool59Not = CmpIOp::create(b, CmpIPredicate::eq, and58, i32Const(0));
+  Value trueConst = ConstantIntOp::create(b, b.getI1Type(), true);
+  Value brCond133 = SelectOp::create(b, cmp57, trueConst, tobool59Not);
 
   Block *ifElse61 = func.addBlock();
   Block *ifThen70 = func.addBlock();
@@ -354,54 +375,54 @@ static FlatSymbolRefAttr makeFp8TruncFunction(Location loc, FloatType outType,
   Block *ifEnd76 = func.addBlock();
   ifEnd76->addArguments({i32, i32}, {loc, loc});
   Value oneConst = i32Const(1);
-  b.create<cf::CondBranchOp>(brCond133, ifElse61, ValueRange{}, ifEnd76,
-                             ValueRange{oneConst, add56});
+  cf::CondBranchOp::create(b, brCond133, ifElse61, ValueRange{}, ifEnd76,
+                           ValueRange{oneConst, add56});
 
   b.setInsertionPointToStart(ifElse61);
   Value tobool63Not =
-      b.create<CmpIOp>(CmpIPredicate::ugt, add56, i32Const((1 << 24) - 1));
+      CmpIOp::create(b, CmpIPredicate::ugt, add56, i32Const((1 << 24) - 1));
   Value incConst = i32Const((1 << (eBits - 1)) + 2);
-  Value inc = b.create<AddIOp>(add41, incConst);
-  Value f8Exponent0 = b.create<SelectOp>(tobool63Not, inc, sub43);
-  Value cmp69 = b.create<CmpIOp>(CmpIPredicate::sgt, f8Exponent0,
-                                 i32Const((1 << eBits) - 1));
-  b.create<cf::CondBranchOp>(cmp69, ifThen70, ValueRange{}, ifEnd71,
-                             ValueRange{});
+  Value inc = AddIOp::create(b, add41, incConst);
+  Value f8Exponent0 = SelectOp::create(b, tobool63Not, inc, sub43);
+  Value cmp69 = CmpIOp::create(b, CmpIPredicate::sgt, f8Exponent0,
+                               i32Const((1 << eBits) - 1));
+  cf::CondBranchOp::create(b, cmp69, ifThen70, ValueRange{}, ifEnd71,
+                           ValueRange{});
 
   b.setInsertionPointToStart(ifThen70);
-  Value ir5 = b.create<TruncIOp>(i8, ir1);
-  Value conv =
-      b.create<OrIOp>(ir5, b.create<ConstantIntOp>(127, b.getI8Type()));
-  Value convOut = b.create<BitcastOp>(outType, conv);
-  b.create<cf::BranchOp>(ret, convOut);
+  Value ir5 = TruncIOp::create(b, i8, ir1);
+  Value c127 = ConstantIntOp::create(b, i8, 127);
+  Value conv = OrIOp::create(b, ir5, c127);
+  Value convOut = BitcastOp::create(b, outType, conv);
+  cf::BranchOp::create(b, ret, convOut);
 
   b.setInsertionPointToStart(ifEnd71);
-  Value shr65 = b.create<ExtUIOp>(i32, tobool63Not);
-  Value mantissa1 = b.create<ShRUIOp>(add56, shr65);
-  Value cmp72 = b.create<CmpIOp>(CmpIPredicate::eq, f8Exponent0, i32Const(0));
-  Value cmp74 = b.create<CmpIOp>(CmpIPredicate::ult, mantissa1,
-                                 i32Const(1 << (16 + eBits)));
-  Value falseConst = b.create<ConstantIntOp>(false, 1);
-  Value brCond = b.create<SelectOp>(cmp72, cmp74, falseConst);
-  b.create<cf::CondBranchOp>(brCond, ret, ValueRange{outZero}, ifEnd76,
-                             ValueRange{f8Exponent0, mantissa1});
+  Value shr65 = ExtUIOp::create(b, i32, tobool63Not);
+  Value mantissa1 = ShRUIOp::create(b, add56, shr65);
+  Value cmp72 = CmpIOp::create(b, CmpIPredicate::eq, f8Exponent0, i32Const(0));
+  Value cmp74 = CmpIOp::create(b, CmpIPredicate::ult, mantissa1,
+                               i32Const(1 << (16 + eBits)));
+  Value falseConst = ConstantIntOp::create(b, b.getI1Type(), false);
+  Value brCond = SelectOp::create(b, cmp72, cmp74, falseConst);
+  cf::CondBranchOp::create(b, brCond, ret, ValueRange{outZero}, ifEnd76,
+                           ValueRange{f8Exponent0, mantissa1});
 
   b.setInsertionPointToStart(ifEnd76);
   Value f8Exponent015 = ifEnd76->getArgument(0);
   Value shr681In = ifEnd76->getArgument(1);
-  Value shr681 = b.create<ShRUIOp>(shr681In, i32Const(16 + eBits));
-  Value and77 = b.create<AndIOp>(shr681, i32Const((1 << mBits) - 1));
-  Value shl79 = b.create<ShLIOp>(f8Exponent015, i32Const(mBits));
-  Value irOr = b.create<OrIOp>(shl79, shl);
-  Value or80 = b.create<OrIOp>(irOr, and77);
-  Value conv81 = b.create<TruncIOp>(i8, or80);
-  Value conv81Out = b.create<BitcastOp>(outType, conv81);
-  b.create<cf::BranchOp>(ret, ValueRange{conv81Out});
+  Value shr681 = ShRUIOp::create(b, shr681In, i32Const(16 + eBits));
+  Value and77 = AndIOp::create(b, shr681, i32Const((1 << mBits) - 1));
+  Value shl79 = ShLIOp::create(b, f8Exponent015, i32Const(mBits));
+  Value irOr = OrIOp::create(b, shl79, shl);
+  Value or80 = OrIOp::create(b, irOr, and77);
+  Value conv81 = TruncIOp::create(b, i8, or80);
+  Value conv81Out = BitcastOp::create(b, outType, conv81);
+  cf::BranchOp::create(b, ret, ValueRange{conv81Out});
 
   func.push_back(ret);
   b.setInsertionPointToStart(ret);
   Value retVal = ret->getArgument(0);
-  b.create<func::ReturnOp>(retVal);
+  func::ReturnOp::create(b, retVal);
 
   return symbolRef;
 }
@@ -454,44 +475,44 @@ makeOCPFp8TruncFunction(Location loc, FloatType outType, Operation *module) {
   Block *ret = new Block();
   ret->addArgument(i8, loc);
 
-  Value bits = b.create<BitcastOp>(i32, in);
-  Value and22 = b.create<AndIOp>(bits, i32Const((1u << 23u) - 1));
-  Value shr23 = b.create<ShRUIOp>(bits, i32Const(23));
-  Value and24 = b.create<AndIOp>(shr23, i32Const(0xff));
-  Value shr25 = b.create<ShRUIOp>(bits, i32Const(24));
-  Value and26 = b.create<AndIOp>(shr25, i32Const(128));
-  Value shl27 = b.create<ShLIOp>(i32Const(1), eWidth);
-  Value add28 = b.create<AddIOp>(shl27, i32Const(-1));
-  Value shl29 = b.create<ShLIOp>(add28, mWidth);
-  Value add30 = b.create<AddIOp>(shl29, and26);
-  Value shl31 = b.create<ShLIOp>(i32Const(-1), mWidth);
-  Value xor32 = b.create<XOrIOp>(shl31, i32Const(-1));
-  Value add33 = b.create<AddIOp>(add30, xor32);
-  Value cmp34 = b.create<CmpIOp>(CmpIPredicate::eq, mWidth, i32Const(2));
+  Value bits = BitcastOp::create(b, i32, in);
+  Value and22 = AndIOp::create(b, bits, i32Const((1u << 23u) - 1));
+  Value shr23 = ShRUIOp::create(b, bits, i32Const(23));
+  Value and24 = AndIOp::create(b, shr23, i32Const(0xff));
+  Value shr25 = ShRUIOp::create(b, bits, i32Const(24));
+  Value and26 = AndIOp::create(b, shr25, i32Const(128));
+  Value shl27 = ShLIOp::create(b, i32Const(1), eWidth);
+  Value add28 = AddIOp::create(b, shl27, i32Const(-1));
+  Value shl29 = ShLIOp::create(b, add28, mWidth);
+  Value add30 = AddIOp::create(b, shl29, and26);
+  Value shl31 = ShLIOp::create(b, i32Const(-1), mWidth);
+  Value xor32 = XOrIOp::create(b, shl31, i32Const(-1));
+  Value add33 = AddIOp::create(b, add30, xor32);
+  Value cmp34 = CmpIOp::create(b, CmpIPredicate::eq, mWidth, i32Const(2));
   Value infNanConst = i32Const(0x7f800000);
-  Value and35 = b.create<AndIOp>(bits, infNanConst);
-  Value cmp36 = b.create<CmpIOp>(CmpIPredicate::eq, and35, infNanConst);
+  Value and35 = AndIOp::create(b, bits, infNanConst);
+  Value cmp36 = CmpIOp::create(b, CmpIPredicate::eq, and35, infNanConst);
 
   Block *bb1 = func.addBlock();
   Block *bb2 = func.addBlock();
   Block *bb3 = func.addBlock();
   Block *bb4 = func.addBlock();
-  b.create<cf::CondBranchOp>(cmp36, bb1, bb4);
+  cf::CondBranchOp::create(b, cmp36, bb1, bb4);
 
   b.setInsertionPointToStart(bb1);
-  Value cmp37 = b.create<CmpIOp>(CmpIPredicate::eq, mWidth, i32Const(3));
-  b.create<cf::CondBranchOp>(cmp37, bb2, bb3);
+  Value cmp37 = CmpIOp::create(b, CmpIPredicate::eq, mWidth, i32Const(3));
+  cf::CondBranchOp::create(b, cmp37, bb2, bb3);
 
   b.setInsertionPointToStart(bb2);
-  Value trunc38 = b.create<TruncIOp>(i8, add33);
-  b.create<cf::BranchOp>(ret, trunc38);
+  Value trunc38 = TruncIOp::create(b, i8, add33);
+  cf::BranchOp::create(b, ret, trunc38);
 
   // This block is a later edit, thus the numbers don't fit with bb6, etc.
   b.setInsertionPointToStart(bb3);
-  Value cmp39 = b.create<CmpIOp>(CmpIPredicate::eq, and22, i32Const(0));
-  Value select40 = b.create<SelectOp>(cmp39, add30, add33);
-  Value trunc41 = b.create<TruncIOp>(i8, select40);
-  b.create<cf::BranchOp>(ret, trunc41);
+  Value cmp39 = CmpIOp::create(b, CmpIPredicate::eq, and22, i32Const(0));
+  Value select40 = SelectOp::create(b, cmp39, add30, add33);
+  Value trunc41 = TruncIOp::create(b, i8, select40);
+  cf::BranchOp::create(b, ret, trunc41);
 
   Block *bb5 = func.addBlock();
   Block *bb6 = func.addBlock();
@@ -499,38 +520,38 @@ makeOCPFp8TruncFunction(Location loc, FloatType outType, Operation *module) {
   SmallVector<int32_t> caseLabels({0, std::numeric_limits<int32_t>::max()});
   SmallVector<Block *> caseSuccessors({ret, bb5});
   SmallVector<ValueRange> caseOperands({ValueRange{i8Const(0)}, ValueRange{}});
-  b.create<cf::SwitchOp>(loc, bits, bb6, ValueRange{}, caseLabels,
-                         caseSuccessors, caseOperands);
+  cf::SwitchOp::create(b, loc, bits, bb6, ValueRange{}, caseLabels,
+                       caseSuccessors, caseOperands);
 
   b.setInsertionPointToStart(bb5);
-  b.create<cf::BranchOp>(ret, i8Const(-128));
+  cf::BranchOp::create(b, ret, i8Const(-128));
 
   Block *bb7 = func.addBlock();
   Block *bb8 = func.addBlock();
   b.setInsertionPointToStart(bb6);
-  Value add41 = b.create<AddIOp>(eWidth, i32Const(-1));
-  Value shl42 = b.create<ShLIOp>(i32Const(-1), add41);
-  Value cmp43 = b.create<CmpIOp>(CmpIPredicate::eq, and24, i32Const(0));
-  Value cmp44 = b.create<CmpIOp>(CmpIPredicate::ne, and22, i32Const(0));
-  Value and45 = b.create<AndIOp>(cmp44, cmp43);
-  b.create<cf::CondBranchOp>(and45, bb7, bb8);
+  Value add41 = AddIOp::create(b, eWidth, i32Const(-1));
+  Value shl42 = ShLIOp::create(b, i32Const(-1), add41);
+  Value cmp43 = CmpIOp::create(b, CmpIPredicate::eq, and24, i32Const(0));
+  Value cmp44 = CmpIOp::create(b, CmpIPredicate::ne, and22, i32Const(0));
+  Value and45 = AndIOp::create(b, cmp44, cmp43);
+  cf::CondBranchOp::create(b, and45, bb7, bb8);
 
   Block *bb9 = func.addBlock();
   bb9->addArgument(i32, loc);
   bb9->addArgument(i32, loc);
   bb9->addArgument(i32, loc);
   b.setInsertionPointToStart(bb7);
-  Value add46 = b.create<AddIOp>(shl42, i32Const(128));
-  b.create<cf::BranchOp>(bb9, ValueRange{i32Const(-126), add46, and22});
+  Value add46 = AddIOp::create(b, shl42, i32Const(128));
+  cf::BranchOp::create(b, bb9, ValueRange{i32Const(-126), add46, and22});
 
   b.setInsertionPointToStart(bb8);
-  Value add47 = b.create<AddIOp>(shl42, i32Const(2));
-  Value add48 = b.create<AddIOp>(and24, i32Const(-127));
-  Value cmp49 = b.create<CmpIOp>(CmpIPredicate::sgt, add48, add47);
-  Value sub50 = b.create<SubIOp>(add47, add48);
-  Value or51 = b.create<OrIOp>(and22, i32Const(0x800000));
-  Value select52 = b.create<SelectOp>(cmp49, i32Const(0), sub50);
-  b.create<cf::BranchOp>(bb9, ValueRange{add48, select52, or51});
+  Value add47 = AddIOp::create(b, shl42, i32Const(2));
+  Value add48 = AddIOp::create(b, and24, i32Const(-127));
+  Value cmp49 = CmpIOp::create(b, CmpIPredicate::sgt, add48, add47);
+  Value sub50 = SubIOp::create(b, add47, add48);
+  Value or51 = OrIOp::create(b, and22, i32Const(0x800000));
+  Value select52 = SelectOp::create(b, cmp49, i32Const(0), sub50);
+  cf::BranchOp::create(b, bb9, ValueRange{add48, select52, or51});
 
   Block *bb10 = func.addBlock();
   Block *bb11 = func.addBlock();
@@ -540,29 +561,29 @@ makeOCPFp8TruncFunction(Location loc, FloatType outType, Operation *module) {
   Value bb9arg0 = bb9->getArgument(0);
   Value bb9arg1 = bb9->getArgument(1);
   Value bb9arg2 = bb9->getArgument(2);
-  Value sub56 = b.create<SubIOp>(i32Const(23), mWidth);
-  Value add57 = b.create<AddIOp>(bb9arg1, sub56);
-  Value min58 = b.create<MinUIOp>(add57, i32Const(31));
-  Value shl59 = b.create<ShLIOp>(i32Const(-1), min58);
-  Value xor60 = b.create<XOrIOp>(shl59, i32Const(-1));
-  Value and61 = b.create<AndIOp>(bb9arg2, xor60);
-  Value add62 = b.create<AddIOp>(add57, i32Const(-1));
-  Value min63 = b.create<MinUIOp>(add62, i32Const(31));
-  Value shl64 = b.create<ShLIOp>(i32Const(1), min63);
-  Value cmp65 = b.create<CmpIOp>(CmpIPredicate::eq, and61, shl64);
-  Value cmp66 = b.create<CmpIOp>(CmpIPredicate::sgt, bb9arg1, i32Const(0));
-  b.create<cf::CondBranchOp>(cmp66, bb10, bb11);
+  Value sub56 = SubIOp::create(b, i32Const(23), mWidth);
+  Value add57 = AddIOp::create(b, bb9arg1, sub56);
+  Value min58 = MinUIOp::create(b, add57, i32Const(31));
+  Value shl59 = ShLIOp::create(b, i32Const(-1), min58);
+  Value xor60 = XOrIOp::create(b, shl59, i32Const(-1));
+  Value and61 = AndIOp::create(b, bb9arg2, xor60);
+  Value add62 = AddIOp::create(b, add57, i32Const(-1));
+  Value min63 = MinUIOp::create(b, add62, i32Const(31));
+  Value shl64 = ShLIOp::create(b, i32Const(1), min63);
+  Value cmp65 = CmpIOp::create(b, CmpIPredicate::eq, and61, shl64);
+  Value cmp66 = CmpIOp::create(b, CmpIPredicate::sgt, bb9arg1, i32Const(0));
+  cf::CondBranchOp::create(b, cmp66, bb10, bb11);
 
   b.setInsertionPointToStart(bb10);
-  Value min67 = b.create<MinUIOp>(bb9arg1, i32Const(31));
-  Value shr68 = b.create<ShRUIOp>(bb9arg2, min67);
-  b.create<cf::BranchOp>(bb12, ValueRange{shr68});
+  Value min67 = MinUIOp::create(b, bb9arg1, i32Const(31));
+  Value shr68 = ShRUIOp::create(b, bb9arg2, min67);
+  cf::BranchOp::create(b, bb12, ValueRange{shr68});
 
   b.setInsertionPointToStart(bb11);
-  Value cmp69 = b.create<CmpIOp>(CmpIPredicate::eq, bb9arg1, i32Const(-1));
-  Value zext70 = b.create<ExtUIOp>(i32, cmp69);
-  Value shl71 = b.create<ShLIOp>(bb9arg2, zext70);
-  b.create<cf::BranchOp>(bb12, ValueRange{shl71});
+  Value cmp69 = CmpIOp::create(b, CmpIPredicate::eq, bb9arg1, i32Const(-1));
+  Value zext70 = ExtUIOp::create(b, i32, cmp69);
+  Value shl71 = ShLIOp::create(b, bb9arg2, zext70);
+  cf::BranchOp::create(b, bb12, ValueRange{shl71});
 
   Block *bb13 = func.addBlock();
   Block *bb14 = func.addBlock();
@@ -571,84 +592,84 @@ makeOCPFp8TruncFunction(Location loc, FloatType outType, Operation *module) {
   bb15->addArgument(i32, loc);
   b.setInsertionPointToStart(bb12);
   Value bb12arg0 = bb12->getArgument(0);
-  Value shr73 = b.create<ShRUIOp>(bb12arg0, i32Const(23));
-  Value or74 = b.create<OrIOp>(shr73, i32Const(-2));
-  Value sub75 = b.create<SubIOp>(bb9arg0, shl42);
-  Value add76 = b.create<AddIOp>(sub75, bb9arg1);
-  Value add77 = b.create<AddIOp>(add76, or74);
-  Value shl78 = b.create<ShLIOp>(i32Const(1), sub56);
-  Value add79 = b.create<AddIOp>(shl78, i32Const(-1));
-  Value and80 = b.create<AndIOp>(bb12arg0, shl78);
-  Value cmp81 = b.create<CmpIOp>(CmpIPredicate::eq, and80, i32Const(0));
-  Value select82 = b.create<SelectOp>(cmp65, cmp81, i1Const(false));
-  Value sext83 = b.create<ExtSIOp>(i32, select82);
-  Value add84 = b.create<AddIOp>(bb12arg0, sext83);
+  Value shr73 = ShRUIOp::create(b, bb12arg0, i32Const(23));
+  Value or74 = OrIOp::create(b, shr73, i32Const(-2));
+  Value sub75 = SubIOp::create(b, bb9arg0, shl42);
+  Value add76 = AddIOp::create(b, sub75, bb9arg1);
+  Value add77 = AddIOp::create(b, add76, or74);
+  Value shl78 = ShLIOp::create(b, i32Const(1), sub56);
+  Value add79 = AddIOp::create(b, shl78, i32Const(-1));
+  Value and80 = AndIOp::create(b, bb12arg0, shl78);
+  Value cmp81 = CmpIOp::create(b, CmpIPredicate::eq, and80, i32Const(0));
+  Value select82 = SelectOp::create(b, cmp65, cmp81, i1Const(false));
+  Value sext83 = ExtSIOp::create(b, i32, select82);
+  Value add84 = AddIOp::create(b, bb12arg0, sext83);
   Value stoch = i1Const(false); // Defaulted arguments.
   Value rng = i32Const(0);
-  Value select85 = b.create<SelectOp>(stoch, rng, add84);
-  Value and86 = b.create<AndIOp>(select85, add79);
-  Value add87 = b.create<AddIOp>(and86, bb12arg0);
-  Value cmp88 = b.create<CmpIOp>(CmpIPredicate::ne, add77, i32Const(0));
-  Value and89 = b.create<AndIOp>(add87, i32Const(0x800000));
-  Value cmp90 = b.create<CmpIOp>(CmpIPredicate::eq, and89, i32Const(0));
-  Value select91 = b.create<SelectOp>(cmp88, i1Const(true), cmp90);
-  b.create<cf::CondBranchOp>(select91, bb13, bb15,
-                             ValueRange{i32Const(1), add87});
+  Value select85 = SelectOp::create(b, stoch, rng, add84);
+  Value and86 = AndIOp::create(b, select85, add79);
+  Value add87 = AddIOp::create(b, and86, bb12arg0);
+  Value cmp88 = CmpIOp::create(b, CmpIPredicate::ne, add77, i32Const(0));
+  Value and89 = AndIOp::create(b, add87, i32Const(0x800000));
+  Value cmp90 = CmpIOp::create(b, CmpIPredicate::eq, and89, i32Const(0));
+  Value select91 = SelectOp::create(b, cmp88, i1Const(true), cmp90);
+  cf::CondBranchOp::create(b, select91, bb13, bb15,
+                           ValueRange{i32Const(1), add87});
 
   b.setInsertionPointToStart(bb13);
-  Value and92 = b.create<AndIOp>(add87, i32Const(0x1000000));
-  Value cmp93 = b.create<CmpIOp>(CmpIPredicate::eq, and92, i32Const(0));
-  b.create<cf::CondBranchOp>(cmp93, bb15, ValueRange{add77, add87}, bb14,
-                             ValueRange{});
+  Value and92 = AndIOp::create(b, add87, i32Const(0x1000000));
+  Value cmp93 = CmpIOp::create(b, CmpIPredicate::eq, and92, i32Const(0));
+  cf::CondBranchOp::create(b, cmp93, bb15, ValueRange{add77, add87}, bb14,
+                           ValueRange{});
 
   b.setInsertionPointToStart(bb14);
-  Value shr94 = b.create<ShRUIOp>(add87, i32Const(1));
-  Value add95 = b.create<AddIOp>(add77, i32Const(1));
-  b.create<cf::BranchOp>(bb15, ValueRange{add95, shr94});
+  Value shr94 = ShRUIOp::create(b, add87, i32Const(1));
+  Value add95 = AddIOp::create(b, add77, i32Const(1));
+  cf::BranchOp::create(b, bb15, ValueRange{add95, shr94});
 
   Block *bb16 = func.addBlock();
   Block *bb17 = func.addBlock();
   b.setInsertionPointToStart(bb15);
   Value bb15arg0 = bb15->getArgument(0);
   Value bb15arg1 = bb15->getArgument(1);
-  Value shr98 = b.create<ShRUIOp>(bb15arg1, sub56);
-  Value cmp99 = b.create<CmpIOp>(CmpIPredicate::eq, mWidth, i32Const(3));
-  Value select100 = b.create<SelectOp>(cmp99, i32Const(-1), i32Const(-2));
-  Value add101 = b.create<AddIOp>(select100, shl27);
-  Value cmp102 = b.create<CmpIOp>(CmpIPredicate::sgt, bb15arg0, add101);
-  b.create<cf::CondBranchOp>(cmp102, bb16, bb17);
+  Value shr98 = ShRUIOp::create(b, bb15arg1, sub56);
+  Value cmp99 = CmpIOp::create(b, CmpIPredicate::eq, mWidth, i32Const(3));
+  Value select100 = SelectOp::create(b, cmp99, i32Const(-1), i32Const(-2));
+  Value add101 = AddIOp::create(b, select100, shl27);
+  Value cmp102 = CmpIOp::create(b, CmpIPredicate::sgt, bb15arg0, add101);
+  cf::CondBranchOp::create(b, cmp102, bb16, bb17);
 
   Block *bb19 = func.addBlock();
   bb19->addArgument(i32, loc);
   b.setInsertionPointToStart(bb16);
-  Value select103 = b.create<SelectOp>(cmp34, add30, add33);
-  b.create<cf::BranchOp>(bb19, ValueRange{select103});
+  Value select103 = SelectOp::create(b, cmp34, add30, add33);
+  cf::BranchOp::create(b, bb19, ValueRange{select103});
 
   Block *bb18 = func.addBlock();
   b.setInsertionPointToStart(bb17);
-  Value cmp104 = b.create<CmpIOp>(CmpIPredicate::eq, bb15arg0, i32Const(0));
-  Value cmp105 = b.create<CmpIOp>(CmpIPredicate::eq, shr98, i32Const(0));
-  Value select106 = b.create<SelectOp>(cmp104, cmp105, i1Const(false));
-  b.create<cf::CondBranchOp>(select106, bb19, ValueRange{and26}, bb18,
-                             ValueRange{});
+  Value cmp104 = CmpIOp::create(b, CmpIPredicate::eq, bb15arg0, i32Const(0));
+  Value cmp105 = CmpIOp::create(b, CmpIPredicate::eq, shr98, i32Const(0));
+  Value select106 = SelectOp::create(b, cmp104, cmp105, i1Const(false));
+  cf::CondBranchOp::create(b, select106, bb19, ValueRange{and26}, bb18,
+                           ValueRange{});
 
   b.setInsertionPointToStart(bb18);
-  Value and107 = b.create<AndIOp>(shr98, xor32);
-  Value shl108 = b.create<ShLIOp>(bb15arg0, mWidth);
-  Value or109 = b.create<OrIOp>(shl108, and107);
-  Value or110 = b.create<OrIOp>(or109, and26);
-  b.create<cf::BranchOp>(bb19, ValueRange{or110});
+  Value and107 = AndIOp::create(b, shr98, xor32);
+  Value shl108 = ShLIOp::create(b, bb15arg0, mWidth);
+  Value or109 = OrIOp::create(b, shl108, and107);
+  Value or110 = OrIOp::create(b, or109, and26);
+  cf::BranchOp::create(b, bb19, ValueRange{or110});
 
   b.setInsertionPointToStart(bb19);
   Value bb19arg0 = bb19->getArgument(0);
-  Value trunc112 = b.create<TruncIOp>(i8, bb19arg0);
-  b.create<cf::BranchOp>(ret, ValueRange{trunc112});
+  Value trunc112 = TruncIOp::create(b, i8, bb19arg0);
+  cf::BranchOp::create(b, ret, ValueRange{trunc112});
 
   func.push_back(ret);
   b.setInsertionPointToStart(ret);
   Value retVal = ret->getArgument(0);
-  Value retOut = b.create<BitcastOp>(outType, retVal);
-  b.create<func::ReturnOp>(retOut);
+  Value retOut = BitcastOp::create(b, outType, retVal);
+  func::ReturnOp::create(b, retOut);
 
   return symbolRef;
 }
@@ -695,16 +716,16 @@ void Fp8TruncToCallPattern::rewrite(TruncFOp op, OpAdaptor adaptor,
           .Default([](auto ignored) { return nullptr; });
 
   auto oneToOut = [&](Value f32) -> Value {
-    auto call = rewriter.create<func::CallOp>(loc, func, outElemType, f32);
+    auto call = func::CallOp::create(rewriter, loc, func, outElemType, f32);
     return call.getResult(0);
   };
 
   Type inType = cloneOrReplace(rawInType, rewriter.getF32Type());
   Value in = rawIn;
   if (rawInElemType.getIntOrFloatBitWidth() < 32)
-    in = rewriter.create<arith::ExtFOp>(loc, inType, rawIn);
+    in = arith::ExtFOp::create(rewriter, loc, inType, rawIn);
   else if (rawInElemType.getIntOrFloatBitWidth() > 32)
-    in = rewriter.create<arith::TruncFOp>(loc, inType, rawIn);
+    in = arith::TruncFOp::create(rewriter, loc, inType, rawIn);
 
   auto inVecType = dyn_cast<VectorType>(inType);
   if (!inVecType)
@@ -714,15 +735,15 @@ void Fp8TruncToCallPattern::rewrite(TruncFOp op, OpAdaptor adaptor,
   Value rets = rewriter.createOrFold<vector::SplatOp>(
       loc,
       rewriter.createOrFold<ConstantFloatOp>(
-          loc, APFloat::getZero(outElemType.getFloatSemantics()), outElemType),
+          loc, outElemType, APFloat::getZero(outElemType.getFloatSemantics())),
       retVecType);
   SmallVector<int64_t> strides = computeStrides(inVecType.getShape());
   for (int64_t i = 0, e = inVecType.getNumElements(); i < e; ++i) {
     SmallVector<int64_t> idx = delinearize(i, strides);
     Value scalar =
-        rewriter.create<vector::ExtractOp>(loc, adaptor.getIn(), idx);
+        vector::ExtractOp::create(rewriter, loc, adaptor.getIn(), idx);
     Value truncated = oneToOut(scalar);
-    rets = rewriter.create<vector::InsertOp>(loc, truncated, rets, idx);
+    rets = vector::InsertOp::create(rewriter, loc, truncated, rets, idx);
   }
   return rewriter.replaceOp(op, rets);
 }
