@@ -19,10 +19,11 @@
 // GPU memory
 //
 //===-----------------------------------------------------===//
+#include "mlir/Dialect/Rock/IR/AmdArchDb.h"
+#include "mlir/Dialect/Rock/IR/GetRockInfo.h"
 #include "mlir/Dialect/Rock/IR/Rock.h"
 #include "mlir/Dialect/Rock/IR/TransformMapBuilder.h"
 #include "mlir/Dialect/Rock/Tuning/GridwiseGemmParams.h"
-#include "mlir/Dialect/Rock/utility/AmdArchDb.h"
 #include "mlir/Dialect/Rock/utility/loweringUtils.h"
 #include "mlir/Dialect/Rock/utility/math.h"
 #include "mlir/Dialect/Rock/utility/transformMapUtils.h"
@@ -205,27 +206,28 @@ struct ThreadwiseWriteAllRewritePattern
     if (auto elemVecType = dyn_cast<VectorType>(destElemType)) {
       LLVM_DEBUG(llvm::dbgs() << "ThreadwiseWriteAllOp saves a vector type"
                               << ", skipping swizzle\n");
-      return success();
+      return failure();
     }
     size_t extraIdxCount = op.getExtraIndices().size();
     VectorizationResult vectorRes =
         getMaxVectorization(destView, extraIdxCount);
     int64_t originalVectorLen = vectorRes.max;
+    int64_t elementsWrittenPerThread = math_util::gcd(dataPerThread, vectorLen);
 
-    if (vectorLen <= originalVectorLen) {
+    if (elementsWrittenPerThread <= originalVectorLen) {
       LLVM_DEBUG(llvm::dbgs()
                  << "Original vectorization of 'iter' is " << originalVectorLen
-                 << ", the output swizzle could achieve " << vectorLen
-                 << ", skipping swizzle\n");
-      return success();
+                 << ", the output swizzle could achieve "
+                 << elementsWrittenPerThread << ", skipping swizzle\n");
+      return failure();
     }
     LLVM_DEBUG(llvm::dbgs()
                << "Original vectorization of 'iter' is " << originalVectorLen
-               << ", the output swizzle could achieve " << vectorLen
-               << ", performing swizzle\n");
+               << ", the output swizzle could achieve "
+               << elementsWrittenPerThread << ", performing swizzle\n");
 
     // Get current workitem ID.
-    auto tid = b.create<WorkitemIdOp>(loc, b.getIndexType());
+    auto tid = WorkitemIdOp::create(b, loc, b.getIndexType());
 
     // Allocate LDS for output.
     auto workgroupMemoryAddressSpace = b.getAttr<gpu::AddressSpaceAttr>(
@@ -233,7 +235,7 @@ struct ThreadwiseWriteAllRewritePattern
     auto ldsMemRefOutputType =
         MemRefType::get({ldsRequiredBytes}, b.getI8Type(), AffineMap{},
                         workgroupMemoryAddressSpace);
-    auto ldsBufferOutput = b.create<GpuAllocOp>(loc, ldsMemRefOutputType);
+    auto ldsBufferOutput = GpuAllocOp::create(b, loc, ldsMemRefOutputType);
     auto typedBuffer = viewBufferAs(b, ldsBufferOutput, destType);
 
     // Convert from raw -> dim0PerBlock, dim1PerBlock
@@ -255,15 +257,14 @@ struct ThreadwiseWriteAllRewritePattern
     auto ldsBufferMNToRaw = transform(b, typedBuffer, transformMNToRaw);
 
     // Store C results to LDS.
-    b.create<ThreadwiseWriteAllOp>(loc, convertedC, ldsBufferMNToRaw,
-                                   /*extraViews=*/idToLDS,
-                                   /*extraIndices=*/ValueRange{tid},
-                                   op.getFeatures(), StoreMethod::Set,
-                                   /*forceUnroll=*/forceUnroll,
-                                   /*useIndexDiffs=*/useIndexDiffs);
+    ThreadwiseWriteAllOp::create(b, loc, convertedC, ldsBufferMNToRaw,
+                                 /*extraViews=*/idToLDS,
+                                 /*extraIndices=*/ValueRange{tid},
+                                 StoreMethod::Set,
+                                 /*forceUnroll=*/forceUnroll,
+                                 /*useIndexDiffs=*/useIndexDiffs);
 
     // Load from LDS to registers.
-    int64_t elementsWrittenPerThread = math_util::gcd(dataPerThread, vectorLen);
     int64_t iter = dataPerThread / elementsWrittenPerThread;
     LLVM_DEBUG(llvm::dbgs()
                << "blockSize: " << blockSize
@@ -300,12 +301,12 @@ struct ThreadwiseWriteAllRewritePattern
     auto ldsBufferForLoad = transform(b, typedBuffer, ldsRead);
 
     // LDS barrier.
-    b.create<LDSBarrierOp>(loc);
+    LDSBarrierOp::create(b, loc);
 
-    b.create<ThreadwiseReadIntoOp>(loc, ldsBufferForLoad, finalC,
-                                   b.getArrayAttr({}), ValueRange{tid},
-                                   forceUnroll, useIndexDiffs);
-    b.create<GpuDeallocOp>(loc, ldsBufferOutput);
+    ThreadwiseReadIntoOp::create(b, loc, ldsBufferForLoad, finalC,
+                                 b.getArrayAttr({}), ValueRange{tid},
+                                 forceUnroll, useIndexDiffs);
+    GpuDeallocOp::create(b, loc, ldsBufferOutput);
 
     SmallVector<int64_t, 5> bidGridLengths;
     SmallVector<StringRef, 5> bidGridOrder;
@@ -397,8 +398,7 @@ struct ThreadwiseWriteAllRewritePattern
     b.replaceOpWithNewOp<ThreadwiseWriteAllOp>(
         op, finalC, matC, idToMatrixCMaps,
         /*extraIndices=*/
-        op.getExtraIndices(), op.getFeatures(), op.getStoreMethod(),
-        forceUnroll, useIndexDiffs);
+        op.getExtraIndices(), op.getStoreMethod(), forceUnroll, useIndexDiffs);
     return success();
   }
 };
@@ -428,6 +428,7 @@ void RockOutputSwizzlePass::runOnOperation() {
       std::optional<std::tuple<int64_t, int64_t, ArrayAttr>> maybeBlockInfo =
           getIdToLDS(threadwiseWriteAll, rewriter);
       if (!maybeBlockInfo.has_value()) {
+        LLVM_DEBUG(llvm::dbgs() << "OutputSwizzle skipped due to getIdToLDS\n");
         return;
       }
       std::tie(dim0PerBlock, dim1PerBlock, idToLDS) = maybeBlockInfo.value();
@@ -454,10 +455,7 @@ void RockOutputSwizzlePass::runOnOperation() {
       writes.push_back(threadwiseWriteAll);
     }
   });
-  if (writes.size() > 1) {
-    LLVM_DEBUG(llvm::dbgs() << "More than one ThreadwiseWriteAllOp writes to "
-                               "global memory, skipping pass\n");
-  } else if (writes.empty()) {
+  if (writes.empty()) {
     LLVM_DEBUG(llvm::dbgs() << "No ThreadwiseWriteAllOp writes to "
                                "global memory, skipping pass\n");
   } else {
@@ -467,7 +465,7 @@ void RockOutputSwizzlePass::runOnOperation() {
 
     // TODO: this pass works for one or two dimensions only at the block level
     GreedyRewriteConfig config;
-    config.strictMode = GreedyRewriteStrictness::ExistingOps;
+    config.setStrictness(GreedyRewriteStrictness::ExistingOps);
     if (failed(applyOpPatternsGreedily(writes, std::move(patterns), config))) {
       return signalPassFailure();
     }
